from typing import Any, Iterable, Iterator, MutableMapping, NamedTuple, Tuple, Type, Union

from django.db.models.base import Model

def make_model_tuple(model: Union[Type[Model], str, Tuple[str, str]]) -> Tuple[str, str]: ...
def resolve_callables(mapping: MutableMapping[str, Any]) -> Iterator[Tuple[str, Any]]: ...
def unpickle_named_row(names: Iterable[str], values: Iterable[Any]) -> NamedTuple: ...
def create_namedtuple_class(*names: str) -> Type[NamedTuple]: ...
